%==========================================================================
%
%   Matlab code for numerical integration assignment.
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright Feb 8, 2012

clear all;
clc;

%  Define integration constants.
a = 1;              %  Lower limit.
b = 2;              %  Upper limit.
n = 10;             %  Number of sub-intervals.
h = (b - a)/n;      %  Grid spacing.
c = (b+a)/2;        %  Midpoint of integration.

%  Subinterval sample locations.
x = linspace(a,b,n+1);

%  Fevals along subintervals.
f1 = x;
f2 = x.^2;
f3 = x.^3;
f4 = x.^4;
f5 = sin(x);

%  Number of functions to evaluate.
Nf = 5;

%  Instantiate expected errors.
err_trap = zeros(Nf,1);      %  Trapezoidal rule.
err_simp = zeros(Nf,1);      %  Simpson's rule.
err_mid  = zeros(Nf,1);      %  Midpoint rule.

%  Expected trapezoidal errors.
err_trap(1) = 0;
err_trap(2) = (n/12)*2*h^3;
err_trap(3) = (n/12)*6*c*h^3;
err_trap(4) = (n/12)*12*c^2*h^3;
err_trap(5) = (-n/12)*(sin(c))*h^3;

%  Expected midpoint errors.
err_mid(1) = 0;
err_mid(2) = -(n/24)*2*h^3;
err_mid(3) = -(n/24)*6*c*h^3;
err_mid(4) = -(n/24)*12*c^2*h^3;
err_mid(5) = -(n/24)*(-cos(c))*h^3;

%  Expected Simpson errors.
err_simp(1) = 0;
err_simp(2) = 0;
err_simp(3) = 0;
err_simp(4) = (n/180)*24*h^5;
err_simp(5) = ((n/2)/90)*(sin(c))*h^5;

%  Analytically computed areas.
AREAS    = zeros(Nf,1);
AREAS(1) = 3/2;
AREAS(2) = (1/3)*(b^3 - a^3);
AREAS(3) = (1/4)*(b^4 - a^4);
AREAS(4) = (1/5)*(b^5 - a^5);
AREAS(5) = cos(1) - cos(2);

%  Compute numerical integrals using the trapezoid rule.
A_trap    = zeros(Nf,1);
A_trap(1) = trapezoid_rule(x,f1);
A_trap(2) = trapezoid_rule(x,f2);
A_trap(3) = trapezoid_rule(x,f3);
A_trap(4) = trapezoid_rule(x,f4);
A_trap(5) = trapezoid_rule(x,f5);

%  Compute numerical integrals using the trapezoid rule.
A_simp    = zeros(Nf,1);
A_simp(1) = Simpson_rule(x,f1);
A_simp(2) = Simpson_rule(x,f2);
A_simp(3) = Simpson_rule(x,f3);
A_simp(4) = Simpson_rule(x,f4);
A_simp(5) = Simpson_rule(x,f5);

%  Calculate observed errors.
err_obs_trap = A_trap - AREAS;
err_obs_simp = A_simp - AREAS;

% Display results.
disp('Expected Trapezoidal Error');
disp(err_trap);
disp('Observed Trapezoidal Error');
disp(err_obs_trap)
disp('Expected Simpson Error');
disp(err_simp);
disp('Observed Simpson Error');
disp(err_obs_simp);

%==========================================================================
%  Now compare Trapezoidal against Simpson's rule in error vs n.
%==========================================================================

%  Constants
L = 50;                 %  Number of evaluation points.
n = logspace(0,3,L);    %  Number of sub-intervals per evaluation.
n = round(n);           %  Make sure n is a series of integers.

%  Integration limits.
a = 0;
b = 1;

%  Make sure all n-values are even so that Simpson's rule can work.
idx    = find(mod(n,2) == 1);
n(idx) = n(idx) + 1;

%  True area.
A0 = erf(1); 

%  Instantiate absolute error vectors.
err_simp = zeros(L,1);      %  Simpson's Rule.
err_trap = zeros(L,1);      %  Trapezoidal Rule.
err_left = zeros(L,1);      %  Trapezoidal Rule.

%  Begin looping over sub-intervals.
for i = 1:L
    
    %  Define subintervals.
    x = linspace(a,b,n(i)+1);
    
    %  fevals.
    f = (2/sqrt(pi))*exp(-x.^2);
    
    %  Calculate areas.
    A1 = left_point_rule(x,f);
    A2 = trapezoid_rule(x,f);
    A3 = Simpson_rule(x,f);
    
    %  Compute absolute errors.
    err_left(i) = abs(A1 - A0);
    err_trap(i) = abs(A2 - A0);
    err_simp(i) = abs(A3 - A0);
    
end

%  Plot results.
figure(1)
clf
loglog(n,err_left,'k','linewidth',2);
grid on
hold on
loglog(n,err_trap,'--r','linewidth',2);
loglog(n,err_simp,'-.b','linewidth',2);
xlabel('Sub-Intervals (n)','fontsize',24);
ylabel('Absolute Error |\epsilon|','fontsize',24);
legend('Left Point','Trapezoidal','Simpson',3);
set(gca,'fontsize',18);


%==========================================================================
%  ECE 6340 ONLY: Compute 2D integrals.
%==========================================================================

%  Set up axis limits and grid spacing.
xMin = 0;
xMax = 1;
yMin = 0;
yMax = 3;
dx   = 0.01;
dy   = 0.01;

%  Define samples and generate mesh.
x = xMin : dx : xMax;
y = yMin : dy : yMax;
[X, Y] = meshgrid(x,y);

%  Evaluate functions.
f1 = X.*Y;
f2 = X.^4 .* Y.^4 ;
f3 = exp(X.*Y);

%  Compute volumes.
V1 = trap_int_2D(X,Y,f1);
V2 = trap_int_2D(X,Y,f2);
V3 = trap_int_2D(X,Y,f3);

% Display results.
disp('Computed Volumes');
disp(V1);
disp(V2);
disp(V3);



